/*
 * Samples from the book: BizTalk Server 2000: A Beginner's Guide.
 * ISBN: 0-07-219011-6, published by Osborne/McGraw-Hill
 *
 * (c) 2000,2001 by Clemens F. Vasters. All rights reserved.
 *
 * This source code is intended only as a supplement to the book
 * "BizTalk Server 2000: A Beginner's Guide". The source code
 * may only be used for self-study, educational purposes in 
 * conjunction with a purchased, orginal copy of this book. You must 
 * not license, sell, rent, lease or redistribute this source code or 
 * derivatives thereof and you must not incorporate this source code 
 * or derivatives thereof in commercial or non-commercial software 
 * components or applications, independent of whether they are used 
 * within your own organization or the organization of your employer 
 * or are sold, licensed, rented, leased or otherwise distributed 
 * or disclosed to third parties.
 * 
 * Any use except self-study, educational purposes in conjunction 
 * with a purchased copy of this book requires written permission and, 
 * depending on the intended use, may require a commercial license 
 * from the author.
 *
 * The author can be contacted by email at clemensv@newtelligence.com
 *  
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE. 
 *
 * Project:  MyFunctoidCan
 * Filename: MyFunctoids.h
 * 
 */

#ifndef __MYFUNCTOIDS_H_
#define __MYFUNCTOIDS_H_

#include "resource.h"       // main symbols
#include <bts_sdk_guids.h>

/////////////////////////////////////////////////////////////////////////////
// CMyFunctoids
class ATL_NO_VTABLE CMyFunctoids : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CMyFunctoids, &CLSID_MyFunctoids>,
	public IDispatchImpl<IFunctoid, &IID_IFunctoid, &LIBID_MYFUNCTOIDCANLib>
{
public:
	CMyFunctoids()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_MYFUNCTOIDS)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_CATEGORY_MAP(CMyFunctoids)
     IMPLEMENTED_CATEGORY(CATID_MapEditFunctoids)
	  IMPLEMENTED_CATEGORY(CATID_BIZTALK_COMPONENT)
END_CATEGORY_MAP()

BEGIN_COM_MAP(CMyFunctoids)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IFunctoid)
END_COM_MAP()

// IMyFunctoids
public:
// IFunctoid
	STDMETHOD(get_Version)(LONG * pVersion)
	{
		if (pVersion == NULL)
			return E_POINTER;
			
		*pVersion = 0x0100;
		return S_OK;
	}
	STDMETHOD(get_FunctionsCount)(LONG * plCount)
	{
		if (plCount == NULL)
			return E_POINTER;
		
		*plCount = 1;
		return S_OK;
	}
	STDMETHOD(GetFunctionDescripter)(LONG lIndex, FUNC_CATEGORY * pFuncCategory, SCRIPT_CATEGORY * pScriptCategory, FUNC_TYPE * pFuncType, BSTR * pbstrName, BSTR * pbstrTooltip, LONG * plBitmapID, LONG * plParmCount, LONG * pFuncId)
	{
		if (pFuncCategory == NULL)
			return E_POINTER;
			
		if (pScriptCategory == NULL)
			return E_POINTER;
			
		if (pFuncType == NULL)
			return E_POINTER;
			
		if (pbstrName == NULL)
			return E_POINTER;
			
		if (pbstrTooltip == NULL)
			return E_POINTER;
			
		if (plBitmapID == NULL)
			return E_POINTER;
			
		if (plParmCount == NULL)
			return E_POINTER;
			
		if (pFuncId == NULL)
			return E_POINTER;

		switch ( lIndex )
		{
		case 0:
			{
				*pFuncCategory = FUNC_CATEGORY_DATETIME_FMT;
				*pScriptCategory = SCRIPT_CATEGORY_VBSCRIPT;
				*pFuncType = FUNC_TYPE_SCRIPTOR;
				*pbstrName = SysAllocString(L"Convert EDIFACT Date");
				*pbstrTooltip = SysAllocString(L"Takes the DTA segment's period expression as "
					                           L"first and the format qualifier as second "
											   L"argument and formats the date as XML compliant "
											   L"ISO date");
				*plBitmapID = 300;
				*plParmCount = 2;
				*pFuncId = 1500;				
				return S_OK;
			}
			break;
		default:
			return E_INVALIDARG;
		}
		
	}

	STDMETHOD(GetFunctionParameter)(LONG funcId, LONG lParameter, LONG * plConnectionType)
	{
		if (plConnectionType == NULL)
			return E_POINTER;

		switch ( funcId )
		{
		case 1500:
			if ( lParameter == -1 )
				*plConnectionType = CONNECT_TYPE_ALL_EXCEPT_RECORD;
			else if ( lParameter == 0 )
               *plConnectionType = CONNECT_TYPE_ALL;
			else 
			   *plConnectionType = CONNECT_TYPE_ALL_EXCEPT_RECORD; 
			return S_OK;
		default:
			return E_INVALIDARG;
		}
		
			
		return S_OK;
	}

	STDMETHOD(GetScriptBuffer)(LONG cFuncId, LONG lInputParameters, BSTR * pbstrScriptBuffer)
	{
		HRESULT hRes = E_FAIL;

		if (pbstrScriptBuffer == NULL)
			return E_POINTER;

		switch ( cFuncId )
		{
		case 1500:
			{
				HRSRC resource = FindResource(_Module.GetResourceInstance(),MAKEINTRESOURCE(IDR_CONVERTEDIFACTDATE),_T("SCRIPT"));
				if ( resource ) 
				{
					HGLOBAL block = LoadResource(_Module.GetResourceInstance(),resource);
					if ( block )
					{
						LPSTR script = (LPSTR)LockResource(block);
						DWORD size = SizeofResource(_Module.GetResourceInstance(),resource);

						*pbstrScriptBuffer = SysAllocStringLen(NULL,size+1);
						MultiByteToWideChar(CP_ACP,0,script,size,*pbstrScriptBuffer,size);
						(*pbstrScriptBuffer)[size]=0x00;
						hRes = S_OK;
					}
				}
			}
			break;
		default:
			hRes = E_INVALIDARG;
		}
			
		return hRes;
	}
};

#endif //__MYFUNCTOIDS_H_
