/*
 * Samples from the book: BizTalk Server 2000: A Beginner's Guide.
 * ISBN: 0-07-219011-6, published by Osborne/McGraw-Hill
 *
 * (c) 2000,2001 by Clemens F. Vasters. All rights reserved.
 *
 * This source code is intended only as a supplement to the book
 * "BizTalk Server 2000: A Beginner's Guide". The source code
 * may only be used for self-study, educational purposes in 
 * conjunction with a purchased, orginal copy of this book. You must 
 * not license, sell, rent, lease or redistribute this source code or 
 * derivatives thereof and you must not incorporate this source code 
 * or derivatives thereof in commercial or non-commercial software 
 * components or applications, independent of whether they are used 
 * within your own organization or the organization of your employer 
 * or are sold, licensed, rented, leased or otherwise distributed 
 * or disclosed to third parties.
 * 
 * Any use except self-study, educational purposes in conjunction 
 * with a purchased copy of this book requires written permission and, 
 * depending on the intended use, may require a commercial license 
 * from the author.
 *
 * The author can be contacted by email at clemensv@newtelligence.com
 *  
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE. 
 *
 * Project:  PipelineAIC
 * Filename: PipelineComp.cpp
 * 
 */

#include "stdafx.h"
#include "PipelineAIC.h"
#include "PipelineComp.h"
#include "wininet.h"
#include <string>

/////////////////////////////////////////////////////////////////////////////
// CPipelineComp

CPipelineComp::CPipelineComp()
{
	// initialize the configuration dictionary
	dictConfig.CoCreateInstance(CLSID_CDictionary);
	dictConfig->put_Value(L"Server Address",CComVariant(L"localhost"));
	dictConfig->put_Value(L"Server Port",CComVariant(21));
	dictConfig->put_Value(L"Logon Anonymous",CComVariant(1));
	dictConfig->put_Value(L"Logon User",CComVariant(L""));
	dictConfig->put_Value(L"Logon Password",CComVariant(L""));
	dictConfig->put_Value(L"Target Filename",CComVariant(L"/out%tracking_id%.dat"));
}

STDMETHODIMP CPipelineComp::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IPipelineComponent
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

CComQIPtr<IErrorInfo> CPipelineComp::ReportError(UINT error)
{
	CComPtr<ICreateErrorInfo> createErrorInfo;
	WCHAR wchBuffer[256]=L"";

	CreateErrorInfo(&createErrorInfo);
	createErrorInfo->SetGUID(IID_IPipelineComponent);
	createErrorInfo->SetSource(L"PipelineAIC.PipelineComp");
	LoadStringW(_Module.GetResourceInstance(),error,wchBuffer,sizeof(wchBuffer));
	createErrorInfo->SetDescription(wchBuffer);
	return createErrorInfo;
}

CComQIPtr<IErrorInfo> CPipelineComp::ReportWin32Error(UINT error)
{
	CComPtr<ICreateErrorInfo> createErrorInfo;
	LPVOID lpMsgBuf;
	HMODULE hLib;

	CreateErrorInfo(&createErrorInfo);
	createErrorInfo->SetGUID(IID_IPipelineComponent);
	createErrorInfo->SetSource(L"PipelineAIC.PipelineComp");

	hLib = LoadLibrary(_T("wininet.dll"));
	FormatMessageW( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_FROM_HMODULE |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		hLib,
		(error&0xffff),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPWSTR) &lpMsgBuf,
		0,
		NULL 
	);
	FreeLibrary(hLib);
	createErrorInfo->SetDescription((WCHAR*)lpMsgBuf);
	LocalFree( lpMsgBuf );
	
	return createErrorInfo;
}

STDMETHODIMP CPipelineComp::Execute(IDispatch * pdispOrder, IDispatch * pdispContext, LONG lFlags, LONG * plErrorLevel)
{
	HRESULT hRes;
	CComQIPtr<IDictionary> dictTransport(pdispOrder);
	CComVariant vWorkingData;

	if ( !dictTransport )
	{
		return E_INVALIDARG;
	}

	
	if ( SUCCEEDED( hRes = dictTransport->get_Value(L"working_data", &vWorkingData )))
	{
		WCHAR TempFile[512];
		CComVariant serverAddress(L"");
		CComVariant serverPort(L"");
		CComVariant logonAnonymous(0);
		CComVariant logonUser(L"");
		CComVariant logonPassword(L"");
		CComVariant targetFilename(L"");
		CComVariant trackingId(L"");
		std::wstring outputFilename;
		const LPWSTR trackingIdPattern = L"%tracking_id%";

		
		// obtain configuration from the configuration dictionary
		dictConfig->get_Value(L"Server Address",&serverAddress);
		dictConfig->get_Value(L"Server Port",&serverPort);
		serverPort.ChangeType(VT_UI2);
		dictConfig->get_Value(L"Logon Anonymous",&logonAnonymous);
		logonAnonymous.ChangeType(VT_I4);
		dictConfig->get_Value(L"Logon User",&logonUser);
		dictConfig->get_Value(L"Logon Password",&logonPassword);
		dictConfig->get_Value(L"Target Filename", &targetFilename);
		dictTransport->get_Value(L"tracking_id", &trackingId);
		

		// construct the target filename
		outputFilename = targetFilename.bstrVal;
		std::wstring::size_type tp = outputFilename.find(trackingIdPattern);
		if ( tp != std::wstring::npos )
		{
			outputFilename.erase(tp,wcslen(trackingIdPattern));
			outputFilename.insert(tp,trackingId.bstrVal);
		}
			
		// if the login is ananymous, we use some default values
		// instead of the configured username and password
		if ( logonAnonymous.lVal != 0 )
		{
			logonUser = L"anonymous";
			logonPassword = L"user@tempuri.org";
		}

		// first thing we do is to write the document to a temporary file
		// that we can hand over to the FTP function later
		if ( GetTempFileNameW(L".", L"bzt", 0, TempFile ) != 0 )
		{
			HANDLE hf = CreateFileW(TempFile, GENERIC_WRITE,FILE_SHARE_WRITE,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
			if ( hf )
			{
				DWORD dw;

				if ( WriteFile(hf,vWorkingData.bstrVal,wcslen(vWorkingData.bstrVal)*sizeof(WCHAR),&dw,NULL) == 0 )
				{
					UINT err=GetLastError();
					hRes = HRESULT_FROM_WIN32(err);
					SetErrorInfo(0,ReportWin32Error(err));
				}
				CloseHandle(hf);
			}
			else
			{
				UINT err=GetLastError();
				hRes = HRESULT_FROM_WIN32(err);
				SetErrorInfo(0,ReportWin32Error(err));
			}
			
		}
		else
		{
			UINT err=GetLastError();
			hRes = HRESULT_FROM_WIN32(err);
			SetErrorInfo(0,ReportWin32Error(err));
		}

		// if everything went well, we proceed to push the document to 
		// the destination using the WinInet API
		if ( SUCCEEDED( hRes ) )
		{

			HINTERNET hInternetSession; 
					
			hInternetSession = InternetOpen(
								  _T("Microsoft BizTalk Server 2000"),   // agent
								  INTERNET_OPEN_TYPE_PRECONFIG,      // access
								  NULL,                              // proxy server (default)
								  NULL,                              // defaults
								  0);                                // synchronous

			
			if ( hInternetSession )
			{
                HINTERNET hFTPSession;

				hFTPSession = ::InternetConnectW(
									  hInternetSession,                // Internet handle
									  serverAddress.bstrVal,           // Server we wish to connect to.
									  serverPort.uintVal,              // Use appropriate port.
									  logonUser.bstrVal,               // Username, can be NULL.
									  logonPassword.bstrVal,           // Password, can be NULL.
									  INTERNET_SERVICE_FTP,            // Flag to use FTP services.
									  0,                               // Flags (see SDK docs).
									  (DWORD)0);                       // See SDK docs
				if ( hFTPSession )
				{
					BOOL bResult;

					bResult = ::FtpPutFileW(
								hFTPSession,       // Handle from an InternetConnect call
								TempFile,
								outputFilename.c_str(),
								FTP_TRANSFER_TYPE_BINARY,
								0);

					if ( !bResult )
					{
						UINT err=GetLastError();
						hRes = HRESULT_FROM_WIN32(err);
						SetErrorInfo(0,ReportWin32Error(err));
					}

					// Close connection.
					InternetCloseHandle(hFTPSession);
				}
				else
				{
					UINT err=GetLastError();
					hRes = HRESULT_FROM_WIN32(err);
					SetErrorInfo(0,ReportWin32Error(err));
				}

				// Close connection.
				InternetCloseHandle(hInternetSession);
			}
			else
			{
				UINT err=GetLastError();
				hRes = HRESULT_FROM_WIN32(err);
				SetErrorInfo(0,ReportWin32Error(err));
			}
		}
		DeleteFileW(TempFile);
	}
	else
	{
		SetErrorInfo(0,ReportError(IDS_WORKING_DATA_NOT_FOUND));
	}
	return hRes;
}

STDMETHODIMP CPipelineComp::EnableDesign(BOOL fEnable)
{
	return S_OK;
}

// IPipelineComponentAdmin
STDMETHODIMP CPipelineComp::GetConfigData(IDispatch * * ppDict)
{
	if (ppDict == NULL)
		return E_POINTER;
		
	return dictConfig.QueryInterface(ppDict);
}

STDMETHODIMP CPipelineComp::SetConfigData(IDispatch * pDict)
{
	dictConfig = pDict;
	return S_OK;
}

// IPipelineComponentDescription
STDMETHODIMP CPipelineComp::ValuesRead(VARIANT * pvar)
{
	if (pvar == NULL)
		return E_POINTER;

	int cEntries = 2;
	// allocate the safearray of VARIANTs
	SAFEARRAY*   psa = SafeArrayCreateVector(VT_VARIANT, 0, cEntries);

	// Populate the safearray variants
	VARIANT* pvarT = (VARIANT*)psa->pvData;
	V_VT(pvarT) = VT_BSTR;
	V_BSTR(pvarT) = SysAllocString(L"working_data");
	pvarT++;
	V_VT(pvarT) = VT_BSTR;
	V_BSTR(pvarT) = SysAllocString(L"tracking_id");
		
	// set up the return value to point to the safearray
	V_VT(pvar) = VT_ARRAY | VT_VARIANT;
	V_ARRAY(pvar) = psa;
		
	return S_OK;
}

STDMETHODIMP CPipelineComp::ValuesWritten(VARIANT * pvar)
{
	if (pvar == NULL)
		return E_POINTER;
		
	return S_OK;
}

STDMETHODIMP CPipelineComp::ContextValuesRead(VARIANT * pvar)
{
	if (pvar == NULL)
		return E_POINTER;
		
	return S_OK;
}

// ISpecifyPipelineComponentUI
STDMETHODIMP CPipelineComp::GetPipelineComponentUIProgID(BSTR * pbstrProgID)
{
	if (pbstrProgID == NULL)
		return E_POINTER;
		
	*pbstrProgID = SysAllocString(L"PipelineAIC.PipelineCompConfig");
	return S_OK;
}

STDMETHODIMP CPipelineComp::IsDirty()
{
	return S_OK;
}

STDMETHODIMP CPipelineComp::Load(IStream * pstm)
{
	HRESULT hRes = E_FAIL;
	CComQIPtr<IPersistStream> persistStream;

	persistStream = dictConfig;
	if ( persistStream )
	{
		hRes = persistStream->Load( pstm );
	};
	return hRes;
}
STDMETHODIMP CPipelineComp::Save(IStream * pstm, BOOL fClearDirty)
{
	HRESULT hRes = E_FAIL;
	CComQIPtr<IPersistStream> persistStream;

	persistStream = dictConfig;
	if ( persistStream )
	{
		hRes = persistStream->Save( pstm, fClearDirty );
	};
	return hRes;
}
STDMETHODIMP CPipelineComp::GetSizeMax(_ULARGE_INTEGER * pCbSize)
{
	HRESULT hRes = E_FAIL;
	CComQIPtr<IPersistStream> persistStream;

	persistStream = dictConfig;
	if ( persistStream )
	{
		hRes = persistStream->GetSizeMax( pCbSize );
	};
	return hRes;
}
STDMETHODIMP CPipelineComp::InitNew()
{
	return S_OK;
}

STDMETHODIMP CPipelineComp::GetClassID(GUID * pClassID)
{
	if (pClassID == NULL)
		return E_POINTER;
		
	*pClassID = GetObjectCLSID();
	return S_OK;
}