/*
 * Samples from the book: BizTalk Server 2000: A Beginner's Guide.
 * ISBN: 0-07-219011-6, published by Osborne/McGraw-Hill
 *
 * (c) 2000,2001 by Clemens F. Vasters. All rights reserved.
 *
 * This source code is intended only as a supplement to the book
 * "BizTalk Server 2000: A Beginner's Guide". The source code
 * may only be used for self-study, educational purposes in 
 * conjunction with a purchased, orginal copy of this book. You must 
 * not license, sell, rent, lease or redistribute this source code or 
 * derivatives thereof and you must not incorporate this source code 
 * or derivatives thereof in commercial or non-commercial software 
 * components or applications, independent of whether they are used 
 * within your own organization or the organization of your employer 
 * or are sold, licensed, rented, leased or otherwise distributed 
 * or disclosed to third parties.
 * 
 * Any use except self-study, educational purposes in conjunction 
 * with a purchased copy of this book requires written permission and, 
 * depending on the intended use, may require a commercial license 
 * from the author.
 *
 * The author can be contacted by email at clemensv@newtelligence.com
 *  
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE. 
 *
 * Project:  SwiftParser
 * Filename: Parser.h
 * 
 */

#ifndef __PARSER_H_
#define __PARSER_H_

#include "resource.h"       // main symbols
#include <bts_sdk_guids.h>  // BizTalk CLSIDs, CATIDs and IIDs
#import  <msxml3.dll>       // import for the Microsoft XML 3.0 parser.

using namespace ::MSXML2;

/*
	RecordInfo: consolidated BizTalk Schema information for record
*/
typedef struct tagRecordInfo
{
	_variant_t vStructure;
	_variant_t vTagName;
	_variant_t vTagPosition;
	_variant_t vDelimiterType;
	_variant_t vDelimiterValue;
	_variant_t vFieldOrder;
	_variant_t vEscapeType;
	_variant_t vEscapeValue;
	_variant_t vCountIgnore;
	_variant_t vAppendNewLine;
	_variant_t vSkipCR;
	_variant_t vSkipLF;
	_variant_t vMinOccurs;
	_variant_t vMaxOccurs;
} 
RecordInfo;

/*
	FieldInfo: consolidated BizTalk Schema information for field
*/
typedef struct tagFieldInfo
{
	_variant_t vEDIDataType;
	_variant_t vJustification;
	_variant_t vFormat;
	_variant_t vCodelist;
	_variant_t vPadChar;
	_variant_t vMinLengthWithPad;
	_variant_t vWrapChar;
	_variant_t vRequired;
	_variant_t vDataType;
	_variant_t vMaxLength;
	_variant_t vMinLength;
	_variant_t vPosStart;
	_variant_t vPosEnd;
} 
FieldInfo;


/////////////////////////////////////////////////////////////////////////////
// CParser
class ATL_NO_VTABLE CParser : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CParser, &CLSID_Parser>,
	public IBizTalkParserComponent,
	public ISupportErrorInfo
{
protected:
	CComPtr<IStream> m_pstmInputStream;  // input stream that holds the interchange
	int m_nCodePage;  // flag indicating whether input stream is IBM EBCDIC
	BOOL m_bEnveloped; // flag indicating whether interchange is enveloped
	DWORD m_dwTotalBytesRead; // total number of bytes read from stream
	DWORD m_dwHeaderOffset;   // offset of document from stream beginning 
	DWORD m_dwDocumentLength; // length of document
	DWORD m_dwRecordPos; // current position within record (used in parsing)
	CComPtr<MSXML2::IXMLDOMDocument2> m_pDocSchema;  // XML schema document
	_bstr_t m_sBizTalkPrefix; // XML namespace prefix for biztalk schema
	_bstr_t m_sDataTypesPrefix; // XML namespace prefix for data types
	_bstr_t m_sSchemaName; // name of the XML schema
	CComBSTR m_sSpecName; // name of the document definiton in the config db

	
public:
	CParser():m_nCodePage(1252),m_bEnveloped(FALSE),
		      m_dwTotalBytesRead(0),m_dwHeaderOffset(0),m_dwDocumentLength(0)
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_PARSER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CParser)
	COM_INTERFACE_ENTRY(IBizTalkParserComponent)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
END_COM_MAP()

BEGIN_CATEGORY_MAP(CParser)
	IMPLEMENTED_CATEGORY(CATID_BIZTALK_PARSER)
END_CATEGORY_MAP()

protected:
	HRESULT ParseHeaders(IDictionary * Dict);
	HRESULT ParseBody(MSXML2::IXMLDOMDocument ** ppDoc);
	HRESULT ParseFooters(IDictionary * Dict);
	HRESULT StreamRead(WCHAR * wchBuffer, DWORD cchBuffer, DWORD * pcchRead );
	HRESULT StreamMoveBack(DWORD dwCharacters);
	HRESULT LoadSchema(BSTR bstrDocType, IXMLDOMDocument2 ** ppdoc);
	HRESULT LoadSchemaByName(BSTR bstrDocName, IXMLDOMDocument2 ** ppdoc);
	HRESULT HandleChildren( RecordInfo & ri, IXMLDOMElementPtr currentEntry, IXMLDOMDocumentPtr targetDocument, IXMLDOMNodePtr currentContext );
	HRESULT ParseRecord( RecordInfo & ri, IXMLDOMElementPtr currentEntry, IXMLDOMDocumentPtr targetDocument, IXMLDOMNodePtr currentContext, BOOL * pbRecordAdded );
	HRESULT ParseField( RecordInfo & ri, FieldInfo & fi, IXMLDOMElementPtr currentEntry, IXMLDOMDocumentPtr targetDocument, IXMLDOMNodePtr currentContext, BSTR * pbstrField );
	
public:
// ISupportErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);
// IBizTalkParserComponent
	STDMETHOD(ProbeInterchangeFormat)(IStream * pData, BOOL FromFile, BSTR EnvName, IStream * pReceiptData, BSTR * Format);
	STDMETHOD(GetInterchangeDetails)(IDictionary * Dict);
	STDMETHOD(GroupsExist)(BOOL * GrpsExist);
	STDMETHOD(GetGroupDetails)(IDictionary * Dict);
	STDMETHOD(GetGroupSize)(LONG * GroupSize, BOOL * LastGroup);
	STDMETHOD(GetNextDocument)(IDictionary * Dict, BSTR DocName, BOOL * DocIsValid, BOOL * LastDocument, GeneratedReceiptLevel * ReceiptGenerated, BOOL * DocIsReceipt, BSTR * CorrelationCompProgID);
	STDMETHOD(GetNativeDocumentOffsets)(BOOL * SizeFromXMLDoc, _LARGE_INTEGER * StartOffset, LONG * DocLength);
};

#endif //__PARSER_H_
