/*
 * Samples from the book: BizTalk Server 2000: A Beginner's Guide.
 * ISBN: 0-07-219011-6, published by Osborne/McGraw-Hill
 *
 * (c) 2000,2001 by Clemens F. Vasters. All rights reserved.
 *
 * This source code is intended only as a supplement to the book
 * "BizTalk Server 2000: A Beginner's Guide". The source code
 * may only be used for self-study, educational purposes in 
 * conjunction with a purchased, orginal copy of this book. You must 
 * not license, sell, rent, lease or redistribute this source code or 
 * derivatives thereof and you must not incorporate this source code 
 * or derivatives thereof in commercial or non-commercial software 
 * components or applications, independent of whether they are used 
 * within your own organization or the organization of your employer 
 * or are sold, licensed, rented, leased or otherwise distributed 
 * or disclosed to third parties.
 * 
 * Any use except self-study, educational purposes in conjunction 
 * with a purchased copy of this book requires written permission and, 
 * depending on the intended use, may require a commercial license 
 * from the author.
 *
 * The author can be contacted by email at clemensv@newtelligence.com
 *  
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE. 
 *
 * Project:  SwiftSerializer
 * Filename: Serializer.cpp
 * 
 */
#include "stdafx.h"
#include "SwiftSerializer.h"
#include "Serializer.h"


#define DATATYPES_SCHEMA L"urn:schemas-microsoft-com:datatypes"
#define BIZTALK_SCHEMA L"urn:schemas-microsoft-com:BizTalkServer"


/////////////////////////////////////////////////////////////////////////////
// CSerializer

STDMETHODIMP CSerializer::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IBizTalkSerializerComponent
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}


STDMETHODIMP CSerializer::Init(BSTR srcQual, BSTR srcID, BSTR destQual, BSTR destID, LONG EnvID, IDictionary * pDelimiters, IStream * OutputStream, LONG NumDocs, LONG PortID)
{
	// SWIFT has only a single document inside an envelope and the envelope 
	// can only be written when we see the document, we will memorize all 
	// arguments and wait for the AddDocument call.
	m_srcQual = srcQual;
	m_srcID = srcID;
	m_destQual = destQual;
	m_destID = destID;
	m_EnvID = EnvID;
	m_pDelimiters = pDelimiters;
	m_pstmOutputStream = OutputStream;
	if ( NumDocs != 1 )
	{
		return E_INVALIDARG;
	}
	m_PortID = PortID;
	return S_OK;
}

STDMETHODIMP CSerializer::AddDocument(LONG DocHandle, IDictionary * Transport, BSTR TrackID, LONG ChannelID)
{
	HRESULT hRes;
	IXMLDOMDocument2Ptr schemaDoc;
	CComVariant vDoc;

	if ( SUCCEEDED( hRes = Transport->get_Value(L"working_data", &vDoc)) &&
		 SUCCEEDED( hRes = LoadSchema(ChannelID,&schemaDoc)))
	{
		IXMLDOMDocument2Ptr dataDoc;
        IXMLDOMSchemaCollectionPtr schemaCache;
		IXMLDOMNodePtr targetNamespace;

		dataDoc.CreateInstance(__uuidof(MSXML2::DOMDocument));
		schemaCache.CreateInstance(__uuidof(MSXML2::XMLSchemaCache));
		if ( SUCCEEDED( targetNamespace = schemaDoc->documentElement->attributes->getQualifiedItem(L"target_namespace",L"urn:schemas-microsoft-com:BizTalkServer")))
		{
			schemaCache->add(targetNamespace->text,CComVariant((IDispatch*)schemaDoc));
		}
		else
		{
 			schemaCache->add(L"",CComVariant(schemaDoc));
		}
		
		dataDoc->schemas = CComVariant((IDispatch*)schemaCache);
		if ( dataDoc->loadXML(vDoc.bstrVal))
		{
			if ( SUCCEEDED( hRes = WriteHeader()))
				if ( SUCCEEDED( hRes = WriteBody(dataDoc)))
					hRes = WriteFooter();
		}
		else
		{
			hRes = E_FAIL;
		}
	};
	return hRes;
}

STDMETHODIMP CSerializer::GetInterchangeInfo(BSTR * InterchangeID, LONG * lNumGroups)
{
	if (InterchangeID == NULL)
		return E_POINTER;
		
	if (lNumGroups == NULL)
		return E_POINTER;
		
	*InterchangeID = SysAllocString(L"0001");
	*lNumGroups = 0;
	return S_OK;
}

STDMETHODIMP CSerializer::GetGroupInfo(LONG * NumDocs, _LARGE_INTEGER * GrpStartOffset, LONG * GrpLen)
{
	if (NumDocs == NULL)
		return E_POINTER;
		
	if (GrpStartOffset == NULL)
		return E_POINTER;
		
	if (GrpLen == NULL)
		return E_POINTER;
		
	return E_NOTIMPL;
}

STDMETHODIMP CSerializer::GetDocInfo(LONG * DocHandle, BOOL * SizeFromXMLDoc, _LARGE_INTEGER * DocStartOffset, LONG * DocLen)
{
	if (DocHandle == NULL)
		return E_POINTER;
		
	if (SizeFromXMLDoc == NULL)
		return E_POINTER;
		
	if (DocStartOffset == NULL)
		return E_POINTER;
		
	if (DocLen == NULL)
		return E_POINTER;
		
	*DocLen = m_dwTotalBytesWritten;
	*SizeFromXMLDoc = FALSE;
	DocStartOffset->QuadPart=0;
	*DocHandle = 0;
	return S_OK;
}

HRESULT CSerializer::LoadSchema(LONG ChannelID, IXMLDOMDocument2 ** ppdoc)
{
	HRESULT hRes = S_OK;
	CComPtr<IBizTalkConfig> cfgroot;
	
	if ( SUCCEEDED( hRes = cfgroot.CoCreateInstance(CLSID_BizTalkConfig)))
	{
		CComPtr<IBizTalkChannel> pchn;
		LONG lOutputDocHandle;

		if ( SUCCEEDED( hRes = cfgroot->CreateChannel((IDispatch**)&pchn)) &&
			 SUCCEEDED( hRes = pchn->Load(ChannelID)) &&
			 SUCCEEDED( hRes = pchn->get_OutputDocument(&lOutputDocHandle)))
		{
			CComPtr<IBizTalkDocument> pdoc;
			
			if ( SUCCEEDED( hRes = cfgroot->CreateDocument((IDispatch**)&pdoc)) &&
				 SUCCEEDED( hRes = pdoc->Load(lOutputDocHandle)))
			{
				CComBSTR content;
				CComPtr<IDictionary> pdict;
				CComVariant vDocType;
									
				if ( SUCCEEDED( pdoc->get_PropertySet((IDispatch**)&pdict)) &&
					 SUCCEEDED( pdict->get_Value(L"functional_identifier",&vDocType)) &&
					 SUCCEEDED( pdoc->get_Content(&content)))
				{
					
					IXMLDOMDocument2Ptr schemadoc;

					schemadoc.CreateInstance(__uuidof(MSXML2::DOMDocument));
					if ( schemadoc->loadXML((_bstr_t)content) )
					{
						vDocType.ChangeType(VT_BSTR);
						m_FuncID = vDocType.bstrVal;
						hRes = schemadoc->QueryInterface(ppdoc);
					}
					else
					{
						hRes = E_FAIL;
					}
				}	
			}
		};
	}
	return hRes;
}

HRESULT CSerializer::WriteHeader()
{
	WCHAR wchHeader[128];
	DWORD dwWritten;
	
	swprintf(wchHeader,L"{1:F01%s0000000000}",m_srcID);
	StreamWrite(wchHeader,wcslen(wchHeader),&dwWritten);
	swprintf(wchHeader,L"{2:I%s%sN0000}{4:",m_FuncID,m_destID);
	StreamWrite(wchHeader,wcslen(wchHeader),&dwWritten);
	return S_OK;
};

HRESULT CSerializer::WriteNode( IXMLDOMNodePtr currentEntry )
{
	HRESULT hRes=S_OK;

	// children? schema info? element?
	if ( currentEntry->selectNodes("*")->length > 0 && 
		 currentEntry->definition != NULL && 
		 currentEntry->nodeType == MSXML2::NODE_ELEMENT )
	{
		IXMLDOMNodeListPtr containedElements;
		IXMLDOMElementPtr recordInfo;
		recordInfo = currentEntry->definition->
			selectSingleNode(L"b:RecordInfo");
		if ( recordInfo != NULL )
		{
			BOOL bIsRecord=FALSE;
			
			_variant_t vStructure = recordInfo->getAttribute("structure");
			_variant_t vTagName = recordInfo->getAttribute("tag_name");
			_variant_t vTagPosition = recordInfo->getAttribute("tag_position");
			_variant_t vDelimiterType = recordInfo->getAttribute("delimiter_type");
			_variant_t vDelimiterValue = recordInfo->getAttribute("delimiter_value");
			_variant_t vFieldOrder = recordInfo->getAttribute("field_order");
			_variant_t vEscapeType = recordInfo->getAttribute("escape_type");
			_variant_t vEscapeValue = recordInfo->getAttribute("escape_value");
			_variant_t vCountIgnore = recordInfo->getAttribute("count_ignore");
			_variant_t vAppendNewLine = recordInfo->getAttribute("append_newline");
			
			// a new record begins here if the source tag identifier (tag_name) is set
			if ( vTagName.vt == VT_BSTR )
			{
				CComBSTR recordPrefix;
				DWORD dwWritten;

				StreamWrite(L"\r\n",2,&dwWritten);
				recordPrefix = L":"; recordPrefix += vTagName.bstrVal;	recordPrefix +=L":";
				StreamWrite(recordPrefix,recordPrefix.Length(),&dwWritten);
				m_dwRecordPos = dwWritten;
				bIsRecord = FALSE;
			}

            containedElements = currentEntry->definition->selectNodes(L"element|attribute");
			for (int i=0;hRes == S_OK && i<containedElements->length;i++)
			{
				IXMLDOMElementPtr entry = containedElements->item[i];
				IXMLDOMNodePtr type = entry->attributes->getNamedItem(L"type");

				if ( type != NULL )
				{
					if ( wcscmp(entry->baseName,L"element") == 0 )
					{
						IXMLDOMNodeListPtr children = currentEntry->selectNodes(type->text);
						for ( int k=0;k<children->length;k++)
						{
							WriteNode(children->item[k]);
						}
					}
					else if ( wcscmp(entry->baseName,L"attribute") == 0 )
					{
						IXMLDOMNodePtr child = currentEntry->attributes->getNamedItem(type->text);

						if ( child != NULL )
						{
							WriteNode(child);
						}
					}
				}
			}

			// if this element was a record and there was no an explicit request not to 
			// terminate it with a new line (which is CR/LF in our case), we emit a
			// newline
			if ( bIsRecord && 
				!(vAppendNewLine.vt == VT_BSTR && !wcscmp(vAppendNewLine.bstrVal,L"no")))
			{
				DWORD dwWritten;
				StreamWrite(L"\0x0d\0x0a",2,&dwWritten);
			}
		}
	}
	else if ( currentEntry->definition != NULL && 
			 (currentEntry->nodeType == MSXML2::NODE_ELEMENT || currentEntry->nodeType == MSXML2::NODE_ATTRIBUTE) )
	{
		IXMLDOMElementPtr fieldInfo;
		fieldInfo = currentEntry->definition->selectSingleNode(L"b:FieldInfo");
		if ( fieldInfo != NULL )
		{
			IXMLDOMNodePtr ndDataType;
			IXMLDOMNodePtr ndMaxLength;
			IXMLDOMNodePtr ndMinLength;
			_variant_t vTypedValue;
			
			_bstr_t sText;
			_variant_t vDataType;
			_variant_t vMaxLength;
			_variant_t vMinLength;
			_variant_t vEDIDataType = fieldInfo->getAttribute("edi_datatype");
			_variant_t vJustification = fieldInfo->getAttribute("justification");
			_variant_t vFormat = fieldInfo->getAttribute("format");
			_variant_t vPadChar = fieldInfo->getAttribute("pad_char");
			_variant_t vMinLengthWithPad = fieldInfo->getAttribute("minLength_withPad");
			_variant_t vWrapChar = fieldInfo->getAttribute("wrap_char");
			
			if ( (ndDataType = currentEntry->definition->attributes->
				                  getQualifiedItem(L"datatype",DATATYPES_SCHEMA)) != NULL)
			{
               vDataType = ndDataType->text; 
			}
			if ( (ndMaxLength = currentEntry->definition->attributes->
				                  getQualifiedItem(L"maxLength",DATATYPES_SCHEMA)) != NULL)
			{
				vMaxLength = ndMaxLength->text;
			}
			if ( (ndMinLength = currentEntry->definition->attributes->
				                   getQualifiedItem(L"minLength",DATATYPES_SCHEMA)) != NULL)
			{
				vMinLength = ndMinLength->text;
			}

			vTypedValue = currentEntry->nodeTypedValue;
			if ( vTypedValue.vt == VT_NULL || vTypedValue.vt == VT_EMPTY )
				vTypedValue = currentEntry->text;

			// check whether this is a decimal number. For decimal numbers we need to 
			// replace the decimal sign from "," to "." (can only occur once)
			// Datatypes in question are D0 ... D9, N0 ... N9 and R0 ... R9
			if ( vEDIDataType.vt == VT_BSTR && 
				 (vEDIDataType.bstrVal[0] == 'D' ||vEDIDataType.bstrVal[0] == 'R') &&
				 (vEDIDataType.bstrVal[1] >= '0' && vEDIDataType.bstrVal[1] <= '9') )
			{
				size_t l;
				vTypedValue.ChangeType(VT_BSTR);
				l = wcslen(vTypedValue.bstrVal);
				for (size_t k=0;k<l;k++)
				{
					if (vTypedValue.bstrVal[k]=='.')
					{
						vTypedValue.bstrVal[k]=',';
						break;
					}
				}
			}
			// otherwise test whether the data-type is a date
			else if ( vTypedValue.vt == VT_DATE && 
				      vEDIDataType.vt == VT_BSTR &&
				      (vEDIDataType.bstrVal[0] == 'D' && vEDIDataType.bstrVal[1] == 'T'))
			{
				SYSTEMTIME st;
				WCHAR * wchBuffer;
				size_t l;

                VariantTimeToSystemTime(vTypedValue.date,&st);
				// empty format ? make it "YYMMDD"
				if ( vFormat.vt != VT_BSTR || vFormat.bstrVal[0] == 0x00)
					vFormat="YYMMDD";
				l= wcslen(vFormat.bstrVal);
				wchBuffer=(WCHAR*)calloc((l+1),sizeof(WCHAR));
                for (int k=l-1;k>=0;k--)
				{
					switch(vFormat.bstrVal[k])
					{
					case 'D': wchBuffer[k]=L'0'+st.wDay%10;
						      st.wDay /= 10;
							  break;
					case 'M': wchBuffer[k]=L'0'+st.wMonth%10;
						      st.wMonth /= 10;
							  break;
				    case 'Y': wchBuffer[k]=L'0'+st.wYear%10;
						      st.wYear /= 10;
							  break;
					default:  wchBuffer[k] = vFormat.bstrVal[k];
							  break;
					}
				}
				vTypedValue = wchBuffer;
				free(wchBuffer);
				
            }
			// otherwise test whether the data-type is a time value
			else if ( vTypedValue.vt == VT_DATE &&
				      vEDIDataType.vt == VT_BSTR &&
				      (vEDIDataType.bstrVal[0] == 'T' && vEDIDataType.bstrVal[1] == 'M'))
			{
				SYSTEMTIME st;
				WCHAR * wchBuffer;
				size_t l;

				VariantTimeToSystemTime(vTypedValue.date,&st);
				// empty format ? make it "YYMMDD"
				if ( vFormat.vt != VT_BSTR || vFormat.bstrVal[0] == 0x00)
					vFormat="HHMMSS";
				l= wcslen(vFormat.bstrVal);
				wchBuffer=(WCHAR*)calloc((l+1),sizeof(WCHAR));
                for (int k=l-1;k>=0;k--)
				{
					switch(vFormat.bstrVal[k])
					{
					case 'H': wchBuffer[k]=L'0'+st.wHour%10;
						      st.wHour /= 10;
							  break;
					case 'M': wchBuffer[k]=L'0'+st.wMinute%10;
						      st.wMinute /= 10;
							  break;
				    case 'S': wchBuffer[k]=L'0'+st.wSecond%10;
						      st.wSecond /= 10;
							  break;
					default:  wchBuffer[k] = vFormat.bstrVal[k];
							  break;
					}
				}
				vTypedValue = wchBuffer;
				free(wchBuffer);
            }
			
			vTypedValue.ChangeType(VT_BSTR);
			sText = vTypedValue.bstrVal;
			if ( sText.length() > 0 )
			{
				WCHAR wszQuery[128];
				DWORD dwInternalPads = 0;
				CComBSTR padChar=L" ";
				IXMLDOMElementPtr elementDecl;
				
				LPWSTR pwsz = (wchar_t*)currentEntry->definition->attributes->getNamedItem(L"name")->text;
				swprintf(wszQuery,L"element[@type='%s']|attribute[@type='%s']",pwsz,pwsz);
				elementDecl = currentEntry->parentNode->definition->selectSingleNode(wszQuery);
				
				if ( vMinLengthWithPad.vt != VT_EMPTY && vMinLengthWithPad.vt != VT_NULL )
				{
					vMinLengthWithPad.ChangeType(VT_UI4);
					if ( sText.length() < vMinLengthWithPad.ulVal)
						dwInternalPads = vMinLengthWithPad.ulVal - sText.length();
					if ( vPadChar.vt == VT_BSTR )
						padChar = vPadChar.bstrVal;
				}
				if ( elementDecl != NULL )
				{
					IXMLDOMNodePtr posStart = elementDecl->attributes->getQualifiedItem(L"posStart",BIZTALK_SCHEMA);
					IXMLDOMNodePtr posEnd = elementDecl->attributes->getQualifiedItem(L"posEnd",BIZTALK_SCHEMA);

					if ( posStart != NULL && posEnd != NULL )
					{
						DWORD dw;
						DWORD dwExternalPads = 0;
						DWORD nposStart = _wtol(posStart->text);
						DWORD nposEnd = _wtol(posEnd->text);
					
						dwExternalPads = nposStart - m_dwRecordPos;
						if ( nposEnd - nposStart < sText.length()+dwInternalPads)
						{
							long a;
							dwInternalPads = (a=(nposEnd - nposStart)-sText.length())<0?0:a;
						}
						if ( dwInternalPads > 0 )
						{
							WCHAR * pBuf=(WCHAR*)malloc(dwInternalPads);
							for (dw=0;dw<dwInternalPads;dw++) pBuf[dw]=padChar[0];
							if ( vJustification.vt==VT_BSTR && !wcscmp(vJustification.bstrVal,L"right"))
							{
								StreamWrite(pBuf,dwInternalPads,&dw);								
								m_dwRecordPos += dw;
							}
							StreamWrite((wchar_t*)sText,sText.length(),&dw);
							m_dwRecordPos += dw;
							if ( vJustification.vt==VT_BSTR && !wcscmp(vJustification.bstrVal,L"left"))
							{
								StreamWrite(pBuf,dwInternalPads,&dw);								
								m_dwRecordPos += dw;
							}
							
						}
						else
						{
							StreamWrite((wchar_t*)sText,sText.length(),&dw);
							m_dwRecordPos += dw;
						};
					}
				};		
			};
		}
	}
	return S_OK;
}

HRESULT CSerializer::WriteBody( IXMLDOMDocument2Ptr pdoc )
{
	return WriteNode(pdoc->documentElement);
}

HRESULT CSerializer::WriteFooter( void )
{
	DWORD dwWritten;
	
	StreamWrite(L"\r\n-}",4,&dwWritten);
	return S_OK;
}

HRESULT CSerializer::StreamWrite(WCHAR * wchBuffer, DWORD cchBuffer, DWORD * pcchWritten )
{
	HRESULT hRes;

	if ( m_nCodePage == 1200 )
	{
		DWORD dwWritten;

		if ( SUCCEEDED( hRes = m_pstmOutputStream->Write(wchBuffer,cchBuffer*2,&dwWritten) ))
		{
			*pcchWritten = dwWritten / 2;
			m_dwTotalBytesWritten += dwWritten;
		}
	}
	else 
	{
		BYTE * pbBuf = (BYTE*)malloc(cchBuffer+1);

		WideCharToMultiByte(m_nCodePage,0,wchBuffer,cchBuffer,(char*)pbBuf,cchBuffer,NULL,NULL);
		if ( SUCCEEDED( hRes = m_pstmOutputStream->Write(wchBuffer,cchBuffer,pcchWritten)))
		{
			m_dwTotalBytesWritten += *pcchWritten;
		}
		free(pbBuf);
	}
	return hRes;
}