/*
 * Samples from the book: BizTalk Server 2000: A Beginner's Guide.
 * ISBN: 0-07-219011-6, published by Osborne/McGraw-Hill
 *
 * (c) 2000,2001 by Clemens F. Vasters. All rights reserved.
 *
 * This source code is intended only as a supplement to the book
 * "BizTalk Server 2000: A Beginner's Guide". The source code
 * may only be used for self-study, educational purposes in 
 * conjunction with a purchased, orginal copy of this book. You must 
 * not license, sell, rent, lease or redistribute this source code or 
 * derivatives thereof and you must not incorporate this source code 
 * or derivatives thereof in commercial or non-commercial software 
 * components or applications, independent of whether they are used 
 * within your own organization or the organization of your employer 
 * or are sold, licensed, rented, leased or otherwise distributed 
 * or disclosed to third parties.
 * 
 * Any use except self-study, educational purposes in conjunction 
 * with a purchased copy of this book requires written permission and, 
 * depending on the intended use, may require a commercial license 
 * from the author.
 *
 * The author can be contacted by email at clemensv@newtelligence.com
 *  
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE. 
 *
 * Project:  SwiftSerializer
 * Filename: Serializer.h
 * 
 */

#ifndef __SERIALIZER_H_
#define __SERIALIZER_H_

#include "resource.h"       // main symbols
#import <msxml3.dll>
#include <bts_sdk_guids.h>
#include <biztalkobjectmodel.h>

using namespace ::MSXML2;

/////////////////////////////////////////////////////////////////////////////
// CSerializer
class ATL_NO_VTABLE CSerializer : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CSerializer, &CLSID_Serializer>,
	public ISupportErrorInfo,
	public IBizTalkSerializerComponent
{
protected:
	CComBSTR m_srcQual;
	CComBSTR m_srcID;
	CComBSTR m_destQual;
	CComBSTR m_destID;
	CComBSTR m_FuncID;
	LONG m_EnvID;
	CComPtr<IDictionary> m_pDelimiters;
	BOOL m_bEnveloped; // flag indicating whether interchange is enveloped
	DWORD m_dwTotalBytesWritten;
	CComPtr<IStream> m_pstmOutputStream;
	LONG m_PortID;
	DWORD m_dwRecordPos;
	int m_nCodePage;
public:
	CSerializer():m_nCodePage(1200),m_dwRecordPos(0),m_dwTotalBytesWritten(0)
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SERIALIZER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSerializer)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IBizTalkSerializerComponent)
END_COM_MAP()

BEGIN_CATEGORY_MAP(CParser)
	IMPLEMENTED_CATEGORY(CATID_BIZTALK_SERIALIZER)
END_CATEGORY_MAP()


public:
	HRESULT LoadSchema(LONG ChannelID, IXMLDOMDocument2 ** ppdoc);
	HRESULT WriteHeader( void );
	HRESULT WriteBody( IXMLDOMDocument2Ptr pdoc );
	HRESULT WriteNode( IXMLDOMNodePtr pnode );
	HRESULT WriteFooter( void );
	HRESULT StreamWrite(WCHAR * wchBuffer, DWORD cchBuffer, DWORD * pcchWritten );
	
// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

public:
// IBizTalkSerializerComponent
	STDMETHOD(Init)(BSTR srcQual, BSTR srcID, BSTR destQual, BSTR destID, LONG EnvID, IDictionary * pDelimiters, IStream * OutputStream, LONG NumDocs, LONG PortID);
	STDMETHOD(AddDocument)(LONG DocHandle, IDictionary * Transport, BSTR TrackID, LONG ChannelID);
	STDMETHOD(GetInterchangeInfo)(BSTR * InterchangeID, LONG * lNumGroups);
	STDMETHOD(GetGroupInfo)(LONG * NumDocs, _LARGE_INTEGER * GrpStartOffset, LONG * GrpLen);
	STDMETHOD(GetDocInfo)(LONG * DocHandle, BOOL * SizeFromXMLDoc, _LARGE_INTEGER * DocStartOffset, LONG * DocLen);
};

#endif //__SERIALIZER_H_
